class EtlFailedException(Exception):
    error_name = 'ETL is failed'

    def __init__(self, error_detail):
        self.error_detail = error_detail


class EtlIsRunningException(Exception):
    error_name = 'Etl is running'

    def __init__(self, error_detail):
        self.error_detail = error_detail



from enum import Enum


class EtlStatusEnum(Enum):
    SUCCEED = 1
    FAILED = 0
    RUNNING = 2


class LogDriveType(Enum):
    TIME_WINDOW_1_PARAM_DAY = "time_window_1_param_day"
    TIME_WINDOW_2_PARAM = "time_window_2_param"
    UPSTREAM = "UPSTREAM"
    FILES = "files"



from datetime import datetime

from log_drive.log_drive_common import check_etl_is_running_and_warning, set_log_success
from log_drive.time_windows import get_sorted_unexecuted_for_time_window_2_param, save_log_for_time_window_2_param

etl_name = 'flask_test_etl'
start_datetime = datetime(2024, 6, 10, 0, 0, 0)
time_interval = 1440
target_table_name = 'test_table_name'
drive_type = 'time_window_2_param'

if __name__ == '__main__':
    # check数据是否正在运行中
    check_etl_is_running_and_warning(etl_name)

    # 获取待执行的datatime tuple list
    datetime_list = get_sorted_unexecuted_for_time_window_2_param(etl_name, start_datetime, time_interval)

    for datetime_tmp in datetime_list:
        log_id = save_log_for_time_window_2_param(etl_name, datetime_tmp, target_table_name, drive_type)
        print('etl process ...')
        set_log_success(log_id)





from sqlalchemy import Column, Integer, String, DateTime, Text

from common.nimbletl_db import Base


class LogDriveTable(Base):
    __tablename__ = 'log_drive_table'

    id = Column(Integer, primary_key=True, autoincrement=True)
    etl_name = Column(String(255), nullable=False)
    target_table_name = Column(String(255), nullable=False)
    drive_type = Column(String(255), nullable=False)
    drive_value = Column(String(255), nullable=False)
    process_start_time = Column(DateTime, nullable=False)
    process_end_time = Column(DateTime)
    etl_result = Column(Integer, nullable=False)
    business_time = Column(Text)

    def __repr__(self):
        return f"<LogDriveTable(id={self.id}, etl_name={self.etl_name}, target_table_name={self.target_table_name}, " \
               f"drive_type={self.drive_type}, drive_value={self.drive_value}, process_start_time={self.process_start_time}, " \
               f"process_end_time={self.process_end_time}, etl_result={self.etl_result}, business_time={self.business_time})>"



from datetime import date, datetime
from typing import List

from common.nimble_enum import LogDriveType
from log_drive.files import get_sorted_unexecuted_file_list
from log_drive.log_drive_common import set_log_success
from log_drive.time_windows import get_sorted_unexecuted_for_time_window_1_param_day, \
    get_sorted_unexecuted_for_time_window_2_param, save_log_for_time_window_2_param
from log_drive.upstream import get_sorted_unexecuted_upstream_job_list

"""
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (1, 'A_etl', 'a_target_name', 'time_window_1_param_day', '2024-04-01', '2024-05-29 17:34:00', '2024-05-29 17:34:04', 1, '2024-03-12, 2024-02-13');
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (2, 'A_etl', 'a_target_name', 'time_window_1_param_day', '2024-04-03', '2024-05-29 17:34:03', '2024-05-29 17:34:06', 1, '2024-03-13, 2024-02-14');
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (3, 'B_etl', 'b_target_name', 'time_window_2_param', '2024-05-01 00:00:00, 2024-05-03 00:00:00', '2024-05-29 18:43:32', '2024-05-29 18:43:34', 1, '2024-03-14, 2024-02-14');
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (4, 'C_etl', 'c_target_name', 'files', 'a_20240404.xlsx', '2024-06-06 18:41:04', '2024-06-06 18:41:07', 1, '2024-03-15, 2024-02-15');
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (5, 'D_etl', 'd_target_name', 'upstream', 'A_etl|1', '2024-06-07 11:35:45', '2024-06-07 11:35:47', 1, '2024-03-16, 2024-02-16');
INSERT INTO nimbletl.log_drive_table (id, etl_name, target_table_name, drive_type, drive_value, process_start_time, process_end_time, etl_result, business_time) VALUES (6, 'A_etl', 'a_target_name', 'time_window_1_param_day', '2024-04-02', '2024-05-29 17:34:03', '2024-05-29 17:34:06', 1, '2024-03-17, 2024-02-17');

"""


def test_get_sorted_unexecuted_for_time_window_1_param_day():
    etl_name = 'A_etl'
    start_date = date(2024, 3, 26)
    result = get_sorted_unexecuted_for_time_window_1_param_day(etl_name, start_date)
    print(result)


def test_get_sorted_unexecuted_for_time_window_2_param():
    etl_name = 'B_etl'
    start_datetime = datetime.strptime('2024-05-01 00:00:00', "%Y-%m-%d %H:%M:%S")
    time_interval = 2880
    result = get_sorted_unexecuted_for_time_window_2_param(etl_name, start_datetime, time_interval)
    print(result)


def test_get_sorted_unexecuted_file_list():
    def get_file_list() -> List[str]:
        return ['a_20240403.xlsx', 'a_20240404.xlsx', 'a_20240405.xlsx']

    etl_name = 'C_etl'
    result = get_sorted_unexecuted_file_list(get_file_list, etl_name)
    print(result)


def test_get_sorted_unexecuted_upstream_job_list():
    etl_name = 'D_etl'
    upstream_etl_name = 'A_etl'
    result = get_sorted_unexecuted_upstream_job_list(etl_name, upstream_etl_name)
    print(result)


def test_save_log_for_time_window_2_param():
    etl_name = 'test_etl'
    start_datetime = datetime(2024, 6, 10, 15, 30, 0)
    end_datetime = datetime(2024, 6, 11, 15, 30, 0)
    datetime_tmp = (start_datetime, end_datetime)
    target_table_name = ''
    drive_type = LogDriveType.TIME_WINDOW_2_PARAM.value
    id = save_log_for_time_window_2_param(etl_name, datetime_tmp, target_table_name, drive_type)
    print(f'insert id: {id}')
    return id


def test_set_log_success():
    log_id = test_save_log_for_time_window_2_param()
    set_log_success(log_id)


if __name__ == '__main__':
    test_get_sorted_unexecuted_for_time_window_1_param_day()
    test_get_sorted_unexecuted_for_time_window_2_param()
    test_get_sorted_unexecuted_file_list()
    test_get_sorted_unexecuted_upstream_job_list()
    test_save_log_for_time_window_2_param()
    test_set_log_success()



from datetime import datetime

from common.nimbletl_enum import EtlStatusEnum
from common.nimbletl_db import get_dataframe, session
from common.nimbletl_exception import EtlIsRunningException
from log_drive.log_drive_table import LogDriveTable


def check_etl_is_running_and_warning(etl_name: str):
    if _check_etl_is_running(etl_name):
        raise EtlIsRunningException(f'[etl_name] ETL正在执行中...')


def _check_etl_is_running(etl_name: str) -> bool:
    """
    判断当前对应的ETL是否有在执行中
    :return:
    """
    last_etl_job_result_str = f'''
        SELECT etl_result
        FROM log_drive_table
        WHERE etl_name = '{etl_name}'
        order by id desc 
        limit 1;
    '''
    result_df = get_dataframe(last_etl_job_result_str)
    if result_df is None or result_df.empty:
        return False
    return result_df['etl_result'][0] == 2


def set_log_success(log_id: int):
    drive_log_entry = session.query(LogDriveTable).filter_by(id=log_id).first()
    if drive_log_entry:
        drive_log_entry.etl_result = EtlStatusEnum.SUCCEED.value
        drive_log_entry.process_end_time = datetime.now()
        session.commit()
        print(f"set log to succeed with id: {log_id}")


def set_log_failed():
    pass



